#Methods that are used in multiple scripts.
#require "param_ils_helper.rb" # just for tmpdir

require "al_algo_specifics.rb"

def tmpdir()
#	if $user_frank 
		return "/.autofs/csother/ubccsprojectarrow/hutter/tmp"
#	else
#		return "/tmp"
#	end
end

def float_regexp()
	return '[+-]?\d+(?:\.\d+)?(?:[eE][+-]\d+)?'; 
end

unless $work_dir
	File.popen("pwd"){|file| $work_dir = file.gets.chomp}
end
#=== This is to locate runalgo.rb
$al_script_dir_for_cluster ="/.autofs/csother/ubccsprojectarrow/hutter/altuning/scripts/"



###########################################################################
#Perform a 2-sided Wilcoxon paired sign rank test. Return value: [p_value, median < 0]
###########################################################################
def wilcoxon_test(alpha, x, y)
	minlen = [x.length, y.length].min
	
	#=== With less than six observations you can't determine significance, so return right away.
	return [false, -1] if minlen < 6
	
	#=== Compute difference of the two vectors.
	diff = []
	for i in 0...minlen
		diff[i] = x[i] - y[i]
	end

	#=== Construct the file holding the R script.
	r_command_filename = "/tmp/tmp_r_cmd_#{random_number_without_rand}"
	r_result_filename = "/tmp/tmp_r_result_#{random_number_without_rand}"
	array_string = diff.join(", ")
	File.open(r_command_filename, "w"){|f|
		f.puts "diff<-c(#{array_string})"
		f.puts "p_value<-wilcox.test(diff, exact=FALSE)$p.value"
		f.puts "write(p_value, '#{r_result_filename}')"
	}

	#=== Call the R script and read the results from the result file.
	system "R CMD BATCH #{r_command_filename}"
	File.delete(r_command_filename)
	pval = -1
	File.open(r_result_filename){|f| pval = f.gets.chomp.to_f}
	File.delete(r_result_filename)
	dir, cmd_filename = File.split(r_command_filename)
	File.delete(cmd_filename + ".Rout")
	
	print diff.sort{|x,y| x.abs <=> y.abs}.map{|x| x<0 ? "-" : (x==0 ? "0" : "+" )}.join(" ")
	puts "  #{pval}"

	require "stats_ils.rb"
#	puts "Testing [#{x.join(", ")}] vs [#{y.join(", ")}]: pval=#{pval}"
	return [pval < alpha/2, pval]  #two-sided test, so divide alpha by 2.
end

###########################################################################
#Sample from an exponential distribution.
###########################################################################
def generate_samples_from_exponential(median, numSamples)
#=== Simple ruby wrapper that takes the mean of an exponential distribution and generates a sample by calling a built-in function in python.
#One sample works from the command line:	cmd = "python -c \"import random; print random.expovariate(#{1/mean})\""
	mean = 1/Math.log(2) * median
	cmd = "python ../scripts/exponential_samples.pyt #{1/mean} #{numSamples}"
	results = []
	File.popen(cmd){|f| results = f.readlines}
	return results.map{|x|x.chomp.to_f}
end

###########################################################################
# Run a set of commands on the cluster.
###########################################################################
def runCommandsOnCluster(commands, waitToFinish=false, name = "", output=false, priorityclass="eh", pre_command=false) 
	t=Time.now
	datetime = t.strftime("%Y-%m-%d %H-%M-%S") # YYYY-MM-DD HH:MM:SS
	todoFilename = "#{tmpdir}/#{name}-tmpAlgosToRun-#{datetime}-#{random_number_without_rand}".gsub(/ /,"")
	File.open(todoFilename, "w"){|f| 
		for command in commands
			f.puts "#{command}"
		end
	}

	shFilename = "#{tmpdir}/#{name}-tmpShFile-#{datetime}-#{random_number_without_rand}.sh".gsub(/ /,"")
	#puts "todoFilename: #{todoFilename}, shFilename: #{shFilename}"
	File.open(shFilename, "w"){|f|
		f.puts "#!/bin/sh"
		f.puts "echo \"Here's what we know from the SGE environment\""
		f.puts "echo HOME=$HOME"
		f.puts "echo USER=$USER"
		f.puts "echo JOB_ID=$JOB_ID"
		f.puts "echo JOB_NAME=$JOB_NAME"
		f.puts "echo HOSTNAME=$HOSTNAME"
		f.puts "echo SGE_TASK_ID=$SGE_TASK_ID"
#		f.puts "echo STDOUT_FILE=$stdout"
#		f.puts "echo STDERR_FILE=$stderr"
#		f.puts "echo jobin=$jobin"
#		f.puts "echo jobout=$jobout"
#		f.puts "echo joberr=$joberr"
		f.puts "RUBYLIB=/ubc/cs/home/h/hutter/arrowspace/altuning/scripts:/cs/public/lib/pkg/ruby-1.8.2/lib/ruby/site_ruby/1.8:/.autofs/binaries/cspubliclib/pkg/ruby-mysql/mysql-ruby-2.7.1"
		f.puts "export RUBYLIB"
		#/cs/local/bin in the PATH is absolutely necessary for JAVA, get java.lang.NoClassDefFoundError otherwise !!!
		f.puts "PATH=/cs/beta/lib/pkg/sge-6.0u7_1/bin/lx24-x86:/cs/local/bin:/cs/local/generic/bin:/cs/local/bin/pbm+:/usr/local/bin:/usr/bin/X11:/bin:/usr/ucb:/usr/bin:/ubc/cs/home/h/hutter/bin:/ubc/cs/home/h/hutter/bin/ix86linux:/opt/kde3/bin:/opt/gnome/bin:/usr/games:/usr/sbin:/sbin:/usr/lib/java/bin:/cs/public/bin:/cs/public/generic/bin:/cs/public/bin/xwindows:/ubc/cs/home/h/hutter/ruby-scripts:/ubc/cs/home/h/hutter/ruby-scripts/autoparam/:/ubc/cs/home/h/hutter/bioinf/impl/scripts:/ubc/cs/home/h/hutter/lib:/ubc/cs/home/h/hutter/mcmc/bin/:/ubc/cs/home/h/hutter/arrowspace/ParamILS/testbed/scripts:/ubc/cs/home/h/hutter/arrowspace/ParamILS/scripts:/cs/public/lib/pkg/ruby-1.8.2/lib/ruby/site_ruby/1.8:/.autofs/binaries/cspubliclib/pkg/ruby-mysql/mysql-ruby-2.7.1:/ubc/cs/home/h/hutter/ant/bin:$PATH" #$SGE_BINARY_PATH:
#		f.puts "ILM_LICENSE_FILE=/cs/local/generic/lib/pkg/ilog/ilm/access.ilm"
#		f.puts "ILOG_LICENSE_FILE=/cs/local/generic/lib/pkg/ilog/ilm/access.ilm"
		f.puts "export ILOG_LICENSE_FILE=/ubc/cs/home/h/hutter/access.ilm"
		
#		f.puts "PATH = $PATH:$GUROBI_HOME/bin"
#		f.puts "LD_LIBRARY_PATH = $LD_LIBRARY_PATH:$GUROBI_HOME/lib"
		
		#f.puts 'export GUROBI_HOME="/.autofs/csother/ubccsprojectarrow/hutter/mip_configuration/gurobi/gurobi201/linux32"'
		f.puts 'export GUROBI_HOME="/.autofs/csother/ubccsprojectarrow/hutter/mip_configuration/gurobi3/gurobi301/linux32"'
		f.puts 'export PATH="${PATH}:${GUROBI_HOME}/bin"'
		f.puts 'export LD_LIBRARY_PATH="${LD_LIBRARY_PATH}:${GUROBI_HOME}/lib"'
		
		f.puts "export PATH"

#		f.puts "export GRB_LICENSE_FILE=/.autofs/csother/ubccsprojectarrow/hutter/mip_configuration/gurobi/gurobi201/gurobi.lic"
		f.puts "export GRB_LICENSE_FILE=/.autofs/csother/ubccsprojectarrow/hutter/mip_configuration/gurobi3/gurobi301/gurobi.lic"

		if output
			f.puts "#\$ -o /ubc/cs/home/h/hutter/arrowspace/sgeout -e /ubc/cs/home/h/hutter/arrowspace/sgeout"
		else
			f.puts "#\$ -o /dev/null -e /dev/null"
		end
		f.puts "line=`head -n $SGE_TASK_ID #{todoFilename} | tail -1`	# Get line of todoFilename."
		
		#=== Stuff for linking MATLAB executable
# # Mark Schmidt recommends this for MATLAB stuff, but it hurts a lot for debugging and works just fine without, at least for me	f.puts "unset DISPLAY"
#		f.puts "MCR=/ubc/cs/home/h/hutter/arrowspace/MCR/v75"
#		f.puts "export MCR"
#		f.puts "LD_LIBRARY_PATH=$MCR/bin/glnx86:$MCR/runtime/glnx86:$MCR/sys/os/glnx86:$MCR/sys/java/jre/glnx86/jre1.5.0/lib/i386/native_threads:$MCR/sys/java/jre/glnx86/jre1.5.0/lib/i386/server:$MCR/sys/java/jre/glnx86/jre1.5.0/lib/i386/:${LD_LIBRARY_PATH}"

        f.puts "LD_LIBRARY_PATH=/cs/local/generic/lib/pkg/matlab-7.10/sys/os/glnx86:/cs/local/generic/lib/pkg/matlab-7.10/bin/glnx86:/cs/local/generic/lib/pkg/matlab-7.10/extern/lib/glnx86:/cs/local/generic/lib/pkg/matlab-7.10/runtime/glnx86:/cs/local/generic/lib/pkg/matlab-7.10/sys/java/jre/glnx86/jre/lib/i386/native_threads:/cs/local/generic/lib/pkg/matlab-7.10/sys/java/jre/glnx86/jre/lib/i386/client:/cs/local/generic/lib/pkg/matlab-7.10/sys/java/jre/glnx86/jre/lib/i386:${LD_LIBRARY_PATH}"
        		
		f.puts "export LD_LIBRARY_PATH"
#		f.puts "XAPPLRESDIR=$MCR/X11/app-defaults"
#		f.puts "export XAPPLRESDIR"

		if pre_command
			f.puts "echo Calling: #{pre_command}  #output"
			f.puts pre_command
		end
		
		cmd = "cd #{$work_dir};"
		f.puts "echo Calling: #{cmd}  #output"
		f.puts cmd

		#cmd= "pwd"
		#f.puts "echo Calling: #{cmd}  #output"
		#f.puts cmd
		cmd= "$line"
		f.puts "echo Calling: #{cmd}  #output"
		f.puts cmd
	}
	
	if commands.length > 0
		consumables = ""
		consumables += " -l dbheavy=1" if $dbheavy
		consumables += " -l db=1" if $dblight
		consumables += " -l cplex=1" if $cplex
#		consumables += " -l max30=1"
#		sge_cmd = "qsub -cwd -m n -t 1-#{commands.length} -l memheavy=1 -P eh #{consumables} #{shFilename}"
		
sge_cmd = "qsub -S /bin/sh -cwd -m n -t 1-#{commands.length} -P #{priorityclass} #{consumables} #{shFilename}"
	end
	
	puts sge_cmd
	puts shFilename
	jobid = nil
	#=== Start job and remember job id.
	File.popen(sge_cmd){|sge_response|
		line = sge_response.gets
		puts line
		if line =~ /Your job (\d+)\./
			jobid = $1.to_i
		elsif line =~ /Your job (\d+) \(/
			jobid = $1.to_i
		end
	}
##	File.delete(todoFilename)
#	File.delete(shFilename)

	if waitToFinish
	#=== Sleep until job done.
		puts "Waiting for SGE job #{jobid} to finish. TODO filename = #{todoFilename}"
		still_running = true
		time = 1
		while still_running
			time = [time*1.5, 60].min
			sleep(time)
			still_running = false
			File.popen("qstat"){|qstat_output|
				while line = qstat_output.gets
					still_running = true if line =~ /^\s*#{jobid}\s*/
				end
				puts "Waiting for SGE job #{jobid} to finish. TODO filename = #{todoFilename}"
			}
		end
	end
	
#	puts commands
	return [jobid, todoFilename]
end

###########################################################################
# Run a set of commands on the cluster, as a single job (not an array job). For commands that are too fast for the scheduler !
###########################################################################
def runCommandsOnClusterOneJob(commands, waitToFinish=false, name = "", output=false, priorityclass="eh")
	t=Time.now
	datetime = t.strftime("%Y-%m-%d %H-%M-%S") # YYYY-MM-DD HH:MM:SS
	todoFilename = "#{tmpdir}/#{name}-tmpAlgosToRun-#{datetime}-#{random_number_without_rand}".gsub(/ /,"")
	File.open(todoFilename, "w"){|f| 
		for command in commands
			f.puts "#{command}"
		end
	}

	shFilename = "#{tmpdir}/#{name}-tmpShFile-#{datetime}-#{random_number_without_rand}.sh".gsub(/ /,"")
	#puts "todoFilename: #{todoFilename}, shFilename: #{shFilename}"
	File.open(shFilename, "w"){|f|
		f.puts "#!/bin/sh"
		f.puts "echo \"Here's what we know from the SGE environment\""
		f.puts "echo HOME=$HOME"
		f.puts "echo USER=$USER"
		f.puts "echo JOB_ID=$JOB_ID"
		f.puts "echo JOB_NAME=$JOB_NAME"
		f.puts "echo HOSTNAME=$HOSTNAME"
		f.puts "echo SGE_TASK_ID=$SGE_TASK_ID"
#		p $HOSTNAME
#		machine_name = $HOSTNAME.sub(/.cs.ubc.ca/,"")
#		f.puts "echo STDOUT_FILE=$stdout"
#		f.puts "echo STDERR_FILE=$stderr"
#		f.puts "echo jobin=$jobin"
#		f.puts "echo jobout=$jobout"
#		f.puts "echo joberr=$joberr"
		f.puts "RUBYLIB=/ubc/cs/home/h/hutter/arrowspace/altuning/scripts:/cs/public/lib/pkg/ruby-1.8.2/lib/ruby/site_ruby/1.8:/.autofs/binaries/cspubliclib/pkg/ruby-mysql/mysql-ruby-2.7.1"
		f.puts "export RUBYLIB"
		#/cs/local/bin in the PATH is absolutely necessary for JAVA, get java.lang.NoClassDefFoundError otherwise !!!
		f.puts "PATH=/ubc/cs/home/h/hutter/arrowspace/altuning/scripts:/cs/beta/lib/pkg/sge-6.0u7_1/bin/lx24-x86:/cs/local/bin:/cs/local/generic/bin:/cs/local/bin/pbm+:/usr/local/bin:/usr/bin/X11:/bin:/usr/ucb:/usr/bin:/ubc/cs/home/h/hutter/bin:/ubc/cs/home/h/hutter/bin/ix86linux:/opt/kde3/bin:/opt/gnome/bin:/usr/games:/usr/sbin:/sbin:/usr/lib/java/bin:/cs/public/bin:/cs/public/generic/bin:/cs/public/bin/xwindows:/ubc/cs/home/h/hutter/ruby-scripts:/ubc/cs/home/h/hutter/ruby-scripts/autoparam/:/ubc/cs/home/h/hutter/bioinf/impl/scripts:/ubc/cs/home/h/hutter/lib:/ubc/cs/home/h/hutter/mcmc/bin/:/ubc/cs/home/h/hutter/arrowspace/ParamILS/scripts:/cs/public/lib/pkg/ruby-1.8.2/lib/ruby/site_ruby/1.8:/.autofs/binaries/cspubliclib/pkg/ruby-mysql/mysql-ruby-2.7.1:/ubc/cs/home/h/hutter/ant/bin:$SGE_BINARY_PATH:$PATH"
#                f.puts "ILM_LICENSE_FILE=/cs/local/generic/lib/pkg/ilog/ilm/access.ilm"
#		f.puts "ILOG_LICENSE_FILE=/cs/local/generic/lib/pkg/ilog/ilm/access.ilm"
		f.puts "export ILOG_LICENSE_FILE=/ubc/cs/home/h/hutter/access.ilm"

#		f.puts "PATH = $PATH:$GUROBI_HOME/bin"
#		f.puts "LD_LIBRARY_PATH = $LD_LIBRARY_PATH:$GUROBI_HOME/lib"
		
#		f.puts 'export GUROBI_HOME="/.autofs/csother/ubccsprojectarrow/hutter/mip_configuration/gurobi/gurobi201/linux32"'
		f.puts 'export GUROBI_HOME="/.autofs/csother/ubccsprojectarrow/hutter/mip_configuration/gurobi3/gurobi301/linux32"'
		f.puts 'export PATH="${PATH}:${GUROBI_HOME}/bin"'
		f.puts 'export LD_LIBRARY_PATH="${LD_LIBRARY_PATH}:${GUROBI_HOME}/lib"'

		f.puts "export PATH"

#		f.puts "export GRB_LICENSE_FILE=/.autofs/csother/ubccsprojectarrow/hutter/mip_configuration/gurobi/gurobi201/gurobi.lic"
		f.puts "export GRB_LICENSE_FILE=/.autofs/csother/ubccsprojectarrow/hutter/mip_configuration/gurobi3/gurobi301/gurobi.lic"

		if output
			f.puts "#\$ -o /ubc/cs/home/h/hutter/arrowspace/sgeout_onejob -e /ubc/cs/home/h/hutter/arrowspace/sgeout_onejob"
		else
			f.puts "#\$ -o /dev/null -e /dev/null"
		end
		cmd = "cd #{$work_dir};"
		f.puts "echo Calling: #{cmd}  #output"
		f.puts cmd

		f.puts "for ((  i = 0 ;  i <= #{commands.length};  i++  ))"
		f.puts "do"
		f.puts "line=`head -n $i #{todoFilename} | tail -1`	# Get line of todoFilename."

		#cmd= "pwd"
		#f.puts "echo Calling: #{cmd}  #output"
		#f.puts cmd
		cmd= "$line"
		f.puts "echo Calling: #{cmd}  #output"
		f.puts cmd
		
		f.puts "done"
	}
	
	if commands.length > 0
		consumables = ""
		consumables += " -l dbheavy=1" if $dbheavy
		consumables += " -l db=1" if $dblight
		consumables += " -l cplex=1" if $cplex
#		sge_cmd = "qsub -cwd -m n -t 1-#{commands.length} -l memheavy=1 -P eh #{consumables} #{shFilename}"
		sge_cmd = "qsub -cwd -m n -t 1-1 -P #{priorityclass} #{consumables} #{shFilename}"
	end
	
	puts sge_cmd
	puts shFilename
	jobid = nil
	#=== Start job and remember job id.
	File.popen(sge_cmd){|sge_response|
		line = sge_response.gets
		puts line
		if line =~ /Your job (\d+)\./
			jobid = $1.to_i
		elsif line =~ /Your job (\d+) \(/
			jobid = $1.to_i
		end
	}
##	File.delete(todoFilename)
#	File.delete(shFilename)

	if waitToFinish
	#=== Sleep until job done.
		puts "Waiting for SGE job #{jobid} to finish. TODO filename = #{todoFilename}"
		still_running = true
		while still_running
			sleep(10)
			still_running = false
			File.popen("qstat"){|qstat_output|
				while line = qstat_output.gets
					still_running = true if line =~ /^\s*#{jobid}\s*/
				end
				puts "Waiting for SGE job #{jobid} to finish. TODO filename = #{todoFilename}"
			}
		end
	end
	return [jobid, todoFilename]
end


###########################################################################
# Run a set of commands on the cluster, as a single job (not an array job). For commands that are too fast for the scheduler !
###########################################################################
def runCommandsOnClusterOneBigArrayJob(commands, numCmdsPerJob, waitToFinish=false, name = "", output=false, priorityclass="eh")
	t=Time.now
	datetime = t.strftime("%Y-%m-%d %H-%M-%S") # YYYY-MM-DD HH:MM:SS
	todoFilename = "#{tmpdir}/#{name}-tmpAlgosToRun-#{datetime}-#{random_number_without_rand}".gsub(/ /,"")
	File.open(todoFilename, "w"){|f| 
		for command in commands
			f.puts "#{command}"
		end
	}

	shFilename = "#{tmpdir}/#{name}-tmpShFile-#{datetime}-#{random_number_without_rand}.sh".gsub(/ /,"")
	#puts "todoFilename: #{todoFilename}, shFilename: #{shFilename}"
	File.open(shFilename, "w"){|f|
		f.puts "#!/bin/sh"
		f.puts "echo \"Here's what we know from the SGE environment\""
		f.puts "echo HOME=$HOME"
		f.puts "echo USER=$USER"
		f.puts "echo JOB_ID=$JOB_ID"
		f.puts "echo JOB_NAME=$JOB_NAME"
		f.puts "echo HOSTNAME=$HOSTNAME"
		f.puts "echo SGE_TASK_ID=$SGE_TASK_ID"
#		p $HOSTNAME
#		machine_name = $HOSTNAME.sub(/.cs.ubc.ca/,"")
#		f.puts "echo STDOUT_FILE=$stdout"
#		f.puts "echo STDERR_FILE=$stderr"
#		f.puts "echo jobin=$jobin"
#		f.puts "echo jobout=$jobout"
#		f.puts "echo joberr=$joberr"
		f.puts "RUBYLIB=/ubc/cs/home/h/hutter/arrowspace/altuning/scripts:/cs/public/lib/pkg/ruby-1.8.2/lib/ruby/site_ruby/1.8:/.autofs/binaries/cspubliclib/pkg/ruby-mysql/mysql-ruby-2.7.1"
		f.puts "export RUBYLIB"
		#/cs/local/bin in the PATH is absolutely necessary for JAVA, get java.lang.NoClassDefFoundError otherwise !!!
		f.puts "PATH=/cs/beta/lib/pkg/sge-6.0u7_1/bin/lx24-x86:/cs/local/bin:/cs/local/generic/bin:/cs/local/bin/pbm+:/usr/local/bin:/usr/bin/X11:/bin:/usr/ucb:/usr/bin:/ubc/cs/home/h/hutter/bin:/ubc/cs/home/h/hutter/bin/ix86linux:/opt/kde3/bin:/opt/gnome/bin:/usr/games:/usr/sbin:/sbin:/usr/lib/java/bin:/cs/public/bin:/cs/public/generic/bin:/cs/public/bin/xwindows:/ubc/cs/home/h/hutter/ruby-scripts:/ubc/cs/home/h/hutter/ruby-scripts/autoparam/:/ubc/cs/home/h/hutter/bioinf/impl/scripts:/ubc/cs/home/h/hutter/lib:/ubc/cs/home/h/hutter/mcmc/bin/:/ubc/cs/home/h/hutter/arrowspace/ParamILS/testbed/scripts:/ubc/cs/home/h/hutter/arrowspace/ParamILS/scripts:/cs/public/lib/pkg/ruby-1.8.2/lib/ruby/site_ruby/1.8:/.autofs/binaries/cspubliclib/pkg/ruby-mysql/mysql-ruby-2.7.1:/ubc/cs/home/h/hutter/ant/bin:$PATH" #$SGE_BINARY_PATH:
#		f.puts "ILM_LICENSE_FILE=/cs/local/generic/lib/pkg/ilog/ilm/access.ilm"
#		f.puts "ILOG_LICENSE_FILE=/cs/local/generic/lib/pkg/ilog/ilm/access.ilm"
		f.puts "export ILOG_LICENSE_FILE=/ubc/cs/home/h/hutter/access.ilm"

#		f.puts "PATH = $PATH:$GUROBI_HOME/bin"
#		f.puts "LD_LIBRARY_PATH = $LD_LIBRARY_PATH:$GUROBI_HOME/lib"
		
#		f.puts 'export GUROBI_HOME="/.autofs/csother/ubccsprojectarrow/hutter/mip_configuration/gurobi/gurobi201/linux32"'
		f.puts 'export GUROBI_HOME="/.autofs/csother/ubccsprojectarrow/hutter/mip_configuration/gurobi3/gurobi301/linux32"'
		f.puts 'export PATH="${PATH}:${GUROBI_HOME}/bin"'
		f.puts 'export LD_LIBRARY_PATH="${LD_LIBRARY_PATH}:${GUROBI_HOME}/lib"'
		
		f.puts "export PATH"

#		f.puts "export GRB_LICENSE_FILE=/.autofs/csother/ubccsprojectarrow/hutter/mip_configuration/gurobi/gurobi201/gurobi.lic"
		f.puts "export GRB_LICENSE_FILE=/.autofs/csother/ubccsprojectarrow/hutter/mip_configuration/gurobi3/gurobi301/gurobi.lic"

		if output
			f.puts "#\$ -o /ubc/cs/home/h/hutter/arrowspace/sgeout -e /ubc/cs/home/h/hutter/arrowspace/sgeout"
		else
			f.puts "#\$ -o /dev/null -e /dev/null"
		end
		f.puts "line=`head -n $SGE_TASK_ID #{todoFilename} | tail -1`	# Get line of todoFilename."

		#=== Stuff for linking MATLAB executable
# # Mark Schmidt recommends this for MATLAB stuff, but it hurts a lot for debugging and works just fine without, at least for me	f.puts "unset DISPLAY"
		f.puts "MCR=/ubc/cs/home/h/hutter/arrowspace/MCR/v75"
		f.puts "export MCR"
    	f.puts "LD_LIBRARY_PATH=$MCR/bin/glnx86:$MCR/runtime/glnx86:$MCR/sys/os/glnx86:$MCR/sys/java/jre/glnx86/jre1.5.0/lib/i386/native_threads:$MCR/sys/java/jre/glnx86/jre1.5.0/lib/i386/server:$MCR/sys/java/jre/glnx86/jre1.5.0/lib/i386/:${LD_LIBRARY_PATH}"

# For original Matlab code:
        f.puts "LD_LIBRARY_PATH=${LD_LIBRARY_PATH}:/cs/local/generic/lib/pkg/matlab-7.6/bin/glnx86:/cs/local/generic/lib/pkg/matlab-7.10/bin/glnx86"
		
# For Java-wrapped Matlab code as .jar:
		f.puts "LD_LIBRARY_PATH=${LD_LIBRARY_PATH}:/cs/local/generic/lib/pkg/matlab-7.10/runtime/glnx86"

		f.puts "export LD_LIBRARY_PATH"
#		f.puts "XAPPLRESDIR=$MCR/X11/app-defaults"
#		f.puts "export XAPPLRESDIR"

		cmd = "cd #{$work_dir};"
		f.puts "echo Calling: #{cmd}  #output"
		f.puts cmd

		f.puts "for ((  i = ($SGE_TASK_ID-1)*#{numCmdsPerJob}+1 ;  i <= $SGE_TASK_ID*#{numCmdsPerJob};  i++  ))"
		f.puts "do"
		f.puts "line=`head -n $i #{todoFilename} | tail -1`	# Get line of todoFilename."

		#cmd= "pwd"
		#f.puts "echo Calling: #{cmd}  #output"
		#f.puts cmd
		cmd= "$line"
		f.puts "echo Calling: #{cmd}  #output"
		f.puts cmd
		
		f.puts "done"
	}

	if commands.length > 0
		consumables = ""
		consumables += " -l dbheavy=1" if $dbheavy
		consumables += " -l db=1" if $dblight
		consumables += " -l cplex=1" if $cplex
#		sge_cmd = "qsub -cwd -m n -t 1-#{commands.length} -l memheavy=1 -P eh #{consumables} #{shFilename}"

		sge_cmd = "qsub -S /bin/sh -cwd -m n -t 1-#{(commands.length/(numCmdsPerJob+0.0)).ceil} -P #{priorityclass} #{consumables} #{shFilename}"
	end
	
	puts sge_cmd
	puts shFilename
	jobid = nil
	#=== Start job and remember job id.
	File.popen(sge_cmd){|sge_response|
		line = sge_response.gets
		puts line
		if line =~ /Your job (\d+)\./
			jobid = $1.to_i
		elsif line =~ /Your job (\d+) \(/
			jobid = $1.to_i
		end
	}
##	File.delete(todoFilename)
#	File.delete(shFilename)

	if waitToFinish
	#=== Sleep until job done.
		puts "Waiting for SGE job #{jobid} to finish. TODO filename = #{todoFilename}"
		still_running = true
		while still_running
			sleep(10)
			still_running = false
			File.popen("qstat"){|qstat_output|
				while line = qstat_output.gets
					still_running = true if line =~ /^\s*#{jobid}\s*/
				end
				puts "Waiting for SGE job #{jobid} to finish. TODO filename = #{todoFilename}"
			}
		end
	end
	return [jobid, todoFilename]
end



###########################################################################
# Run a set of algorithm runs the simple way (directly on the local machine).
###########################################################################
def runAlgosLocally(algosToRun)
	for algoToRun in algosToRun
		#=== Perform each run as a function call -- can use the *same* DB connection, not open a new one for each run.
		runalgo(algoToRun[0], algoToRun[1], algoToRun[2])
		
		#=== Perform each run from the command line.
		#cmd = "ruby #{$al_script_dir_for_cluster}/al_runalgo.rb #{algoToRun[0]} #{algoToRun[1]} #{algoToRun[2]}"
		#puts "Calling locally: #{cmd}"
		#system cmd
	end
end

###########################################################################
# Run a set of algorithm runs on the cluster.
###########################################################################
def runAlgosOnCluster(algosToRun, waitToFinish=false, algo = "", output=false, priorityclass = "eh", numCommandsPerJob = 1)
	commandsToRun = []
	for algoToRun in algosToRun
	#cd #{$work_dir}; 
		commandsToRun << "/usr/bin/ruby #{$al_script_dir_for_cluster}/al_runalgo.rb #{algoToRun[0]} #{algoToRun[1]} #{algoToRun[2]} #{algoToRun[3]}"
	end
	runCommandsOnClusterOneBigArrayJob(commandsToRun, numCommandsPerJob, waitToFinish, algo, output, priorityclass)
#	if oneJob
#		runCommandsOnClusterOneJob(commandsToRun, waitToFinish, algo, output, priorityclass)		
#	else
#		runCommandsOnCluster(commandsToRun, waitToFinish, algo, output, priorityclass)
#	end
end


###########################################################################
# Run a set of algorithm runs with the jobserver.
###########################################################################
def runAlgosWithJobserver(algosToRun)
	require 'jobserver'

	#=== Put jobs in the queque.
	myJobQueue = []
	for algoToRun in algosToRun
		cmd = "cd #{$work_dir}; /usr/bin/ruby #{$al_script_dir_for_cluster}/al_runalgo.rb #{algoToRun[0]} #{algoToRun[1]} #{algoToRun[2]}"
		myJobQueue << Job.new(:name=>"ALGORUN_CONFIG #{algoToRun[0]}, seed #{algoToRun[1]}, cutofftime #{algoToRun[2]}.", :client_command=>cmd)
	end

	if myJobQueue.length > 0
		#=== Configure jobserver.
		Job.nicelevel = nil # nice doesn't work at UBC
		server = JobServer.new(myJobQueue, $work_dir, 0)

		hosts =  %w{arrow41 arrow42 arrow43 arrow44 arrow45 arrow46 arrow47 arrow48 arrow49 arrow50}
#		hosts =  %w{arrow25}
#		hosts =  %w{arrow36 arrow37 arrow38 arrow39 arrow40 arrow41 arrow42 arrow43 arrow44 arrow45 arrow46 arrow47 arrow48 arrow49}
#		hosts =  %w{arrow41 arrow42 arrow43 arrow44 arrow45 arrow46 arrow47 arrow48 arrow49}
#		hosts =  %w{arrow21 arrow22 arrow24 arrow26 arrow27 arrow30 arrow31 arrow34 arrow36 arrow37 arrow38 arrow39 arrow41 arrow42 arrow43 arrow44 arrow45 arrow46 arrow47 arrow48 arrow49}
#		hosts =  %w{arrow43 arrow44 arrow45 arrow46 arrow47 arrow48 arrow49}
#		hosts =  %w{arrow01 arrow02 arrow03 arrow21 arrow22 arrow23 arrow24 arrow25 arrow27 arrow28 arrow29 arrow30 arrow31 arrow32 arrow33 arrow34 arrow35 arrow36 arrow37 arrow40 arrow41 arrow42 arrow43 arrow44 arrow45 arrow46 arrow47 arrow48}
#		hosts =  %w{arrow30 arrow31 arrow32 arrow33 arrow34 arrow35 arrow36 arrow37 arrow40 arrow41 arrow42 arrow43 arrow44 arrow45 arrow46 arrow47 arrow48}
#		hosts =  %w{arrow01 arrow02 arrow03 arrow04 arrow05 arrow06 arrow07 arrow08 arrow09 arrow10 arrow11 arrow12 arrow13 arrow14 arrow15 arrow16 arrow17 arrow18 arrow19 arrow20 arrow21 arrow22 arrow23 arrow24 arrow25 arrow26 arrow27 arrow28 arrow29 arrow30 arrow31 arrow32 arrow33 arrow34 arrow35 arrow36 arrow37 arrow38 arrow39 arrow40 arrow41 arrow42 arrow43 arrow44 arrow45 arrow46 arrow47 arrow48 arrow49}
		
		while not hosts.empty? # Add workers in randomized order, so not everyone pounds on the last worker.
			randind = (random_number_without_rand*hosts.length).floor
			#randind = rand(hosts.length)
			host = hosts[randind]
			hosts.delete_at(randind)
			server.add_ssh_worker(host, $work_dir, 2)
		end

		#=== Dump out statistics on the progress every 5 seconds. 
		server.dumpStatistics #($statsfile, 5)

		#=== Wait until all jobs have finished.
		server.serve
	end
end


###########################################################################
# Gets the results for running algo on multiple instances - keeps track of which runs are used and does not reuse them !
###########################################################################
def getAlgoResultsForInstsAndParams(algo, instanceAndParamsHash, cutoff_time, cutoff_length, sorted_instances, oncluster=0, db=false)
	puts "getAlgoResultsForInstsAndParams begin"
	puts db
	$stdout.flush

	if algo =~ /gurobi/ or algo =~ /cplex.*11/ or algo =~ /scip/ or !db # or algo =~ /cplex.*12/ or algo =~ /lp/ 
		p algo
		raise "special case"
		cmd_arrays = []
		
		#=== Prepare runs.
		#p instanceAndParamsHash
		for instance in instanceAndParamsHash.keys
			#p instanceAndParamsHash[instance]
			for param_string in instanceAndParamsHash[instance].keys
				params = instanceAndParamsHash[instance][param_string]["params"]
				
				qual = instanceAndParamsHash[instance][param_string]["qual"]
#				rest_of_instance_specific_info = instanceAndParamsHash[instance][param_string]["rest"]
				seeds = instanceAndParamsHash[instance][param_string]["seeds"]
				exec_param_string = params.keys.map{|x| "-#{x} #{params[x]}"}.join(" ")

				instanceAndParamsHash[instance][param_string]["results"] = {}
				for seed in seeds
					successful = false
					
					#if rest_of_instance_specific_info.length == 1 and rest_of_instance_specific_info[0] == "0"
					#	rest_of_instance_specific_info[0] = "instance_specific"
					#end

					#cmd = "#{algo} #{instance} \"#{rest_of_instance_specific_info}\" #{cutoff_time} #{cutoff_length} #{seed} #{exec_param_string}"
					#cmd = "\"#{algo}\" #{instance} instance_specific #{cutoff_time} #{cutoff_length} #{seed} #{exec_param_string}"
#					if db
#						cmd = "\"#{algo}\" #{instance} instance_specific #{cutoff_time} #{cutoff_length} #{seed} #{exec_param_string}"
#					else
						#cmd = "#{algo} #{instance} instance_specific #{cutoff_time} #{cutoff_length} #{seed} #{exec_param_string}"
						#cmd = "#{algo} #{instance} #{rest_of_instance_specific_info[0]} #{cutoff_time} #{cutoff_length} #{seed} #{exec_param_string}"
						cmd = "#{algo} #{instance} #{qual} #{cutoff_time} #{cutoff_length} #{seed} #{exec_param_string}"
						
#					end
					
					cmd_arrays << [cmd, instanceAndParamsHash, instance, param_string]
				end
			end
		end
		
		#=== oncluster=-1: wait until runs done on the cluster.
		if oncluster == "-1" or oncluster == -1
			runCommandsOnCluster(cmd_arrays.map{|cmd_array| "ruby simple_result_wrapper.rb #{cmd_array[0]}"}, true, "eval#{algo}", true) # wait for results.
			oncluster = 0
		end
		
		if oncluster == "0" or oncluster == 0
			#=== Perform the runs locally.

			# Do runs, save results in instanceAndParamsHash, return.
			for cmd_array in cmd_arrays
				cmd = cmd_array[0]
				instanceAndParamsHash = cmd_array[1]
				instance = cmd_array[2]
				param_string = cmd_array[3]
#				p "param_string"
#				p param_string
				
				try = 1
				begin
					t = Time.now
					datetime = t.strftime("%Y-%m-%d %H:%M:%S") # YYYY-MM-DD HH:MM:SS
					algo_output_file = "#{tmpdir}/tmp-#{datetime}-#{random_number_without_rand}".gsub(/ /,"")

					if db
						cmd_via_db = "ruby simple_result_wrapper.rb #{cmd}"
						puts "Executing: #{cmd_via_db} > #{algo_output_file}"
						runresult = system(cmd_via_db + " > #{algo_output_file}")
					else 
						puts "Trial #{try}. Executing cmd: #{cmd} > #{algo_output_file}"
						runresult = system(cmd + " > #{algo_output_file}")
					end

					File.open(algo_output_file){|file|
						while line = file.gets
							if line =~ /Result for ParamILS: /
								line = line.sub(/Result for ParamILS: /,"")
								puts "Result: #{line.strip}"

								if @use_run_log
									runlog cmd
									runlog "Result: #{line.strip}"
									runlog "\n"
								end

								solved, runtime, runlength, best_sol, seed = line.split(",").map!{|x|x.strip}
									
								#puts "\n\nFilling in results for instance #{instance}, param_string #{param_string}, and seed #{seed}\n\n"
								
								instanceAndParamsHash[instance][param_string]["results"][seed.to_i] = [solved, runtime.to_f, runlength.to_i, best_sol.to_f, seed.to_i]
								successful  = true
								break
							end
						end
						raise "No result in result file of #{cmd}:\n#{algo_output_file}" if instanceAndParamsHash[instance][param_string]["results"].empty?
					}
					File.delete(algo_output_file)
						
					raise "Call unsuccessful: #{cmd}" unless successful 
					#raise "Solver crashed on #{cmd}" if instanceAndParamsHash[instance][param_string]["results"][seed.to_i][0] == "CRASHED"
					puts "WARNING: Solver crashed on #{cmd}" if instanceAndParamsHash[instance][param_string]["results"][seed.to_i][0] == "CRASHED"
					raise "No solver result for #{cmd}" unless instanceAndParamsHash[instance][param_string]["results"][seed.to_i][0]
					
=begin				rescue #=== Catch error due to files disappearing or something similar
					if $!.to_s =~ /No such file or directory/ or $!.to_s =~ /Input\output error/ or $!.to_s =~ /Stale NFS file handle/ or (not runresult) or (not successful) or $!.to_s =~ /crashed/ or $!.to_s =~ /No solver result/
						p $!.to_s
						try += 1
						sleep(10)
						if try < 5
							retry 
						else
							raise $1.to_s
						end
#					elsif (not runresult) or (not successful) or $!.to_s =~ /crashed/ or $!.to_s =~ /No solver result/
# 						raise "Solver crashed on #{cmd}. ERROR MESSAGE: #{$!.to_s}"
					else
						raise $1.to_s
			end
=end
				end
			end
			return true #results ready
		else
			#=== Put runs on cluster and retrieve results later.
			#raise "Doesn't make sense to run evaluation on cluster if the results aren't saved anywhere" unless db
			
			#					cmd = cmd.gsub(/"/, "\\\"")
#			runCommandsOnCluster(cmd_arrays.map{|cmd_array| "ruby simple_result_wrapper.rb #{cmd_array[0]}"}, false, "eval#{algo}", true) # wait for results.
			runCommandsOnClusterOneBigArrayJob(cmd_arrays.map{|cmd_array| "ruby simple_result_wrapper.rb #{cmd_array[0]}"}, 15, false, "eval#{algo}", true) # wait for results.
#			syntax: runCommandsOnClusterOneBigArrayJob(commandsToRun, numCommandsPerJob, waitToFinish, algo, output, priorityclass)
			puts "Jobs submitted. Re-run with oncluster=-1 to get results once cluster jobs are done..."
			return false # results not ready
#			exit -1
		end
	end
	
#	raise "want to use db"
	
if $fakedCensoringForExactSameResults
	#========================================= FOR DEBUGGING THE PRUNING (CPU time is not deterministic with the same seed :-( )
	#=== Temporary, for debugging, let everyone use the exact same results from the DB, but keep all entries as if used with the real time.
	real_cutoff_time = cutoff_time
	cutoff_time = [cutoff_time, $fakedCensoringRuntime].max
end

	#=== Collect runs I still have to do.
	algosToRun = []
	for instance in sorted_instances
		next unless instanceAndParamsHash.key?(instance) # If not all the original instances are in the instanceHash
		for param_string in instanceAndParamsHash[instance].keys
			params = instanceAndParamsHash[instance][param_string]["params"]
#			puts "getAlgoResultsForInstsAndParams algorun_config_id. Instance #{instance}, #{params.length} params #{params.keys.join(" ")}"
#			$stdout.flush

			algorun_config_id = get_algorun_config_id(algo, params, instance, instanceAndParamsHash[instance][param_string]["qual"])[0]
#			puts "getAlgoResultsForInstsAndParams after algorun_config_id"
#			$stdout.flush

			instanceAndParamsHash[instance][param_string]["algorun_config_id"] = algorun_config_id
			seeds = instanceAndParamsHash[instance][param_string]["seeds"].map{|x| x.to_i}
			
#			p seeds
			if seeds.length < 100
				#=== Just check whether we have them one by one.
				neededSeeds = seeds
				for seed in seeds
					if is_computed_seed(algorun_config_id, cutoff_time, seed)
						neededSeeds = neededSeeds - [seed]
					end
				end				
#				p neededSeeds
			else	
				computedSeeds = computedSeeds(algorun_config_id, cutoff_time)
				
				#=== Make sure not to use runs that were used before.
				neededSeeds = seeds - computedSeeds # Ruby set difference.
			end
			
			unless neededSeeds.empty?
				for seed in neededSeeds 
					algosToRun << [algorun_config_id, seed, cutoff_time, $global_database]
				end
			end
			#puts "Need #{neededSeeds.length} new runs. (Want #{seeds.length}, have #{computedSeeds.length}, #{(seeds&computedSeeds).length} matching) for algorun_config_id #{algorun_config_id}" 
			puts "Need #{neededSeeds.length} new runs for algorun_config_id #{algorun_config_id}." 
			$stdout.flush
		end
	end
	
	#=== Remove duplicates. (Can't use uniq! because array entries)
	uniqAlgosToRun = []
	conf_seed_hash = Hash.new
	for algoToRun in algosToRun
		conf, seed = algoToRun
		if conf_seed_hash.key?(conf) 
			next if conf_seed_hash[conf].include?(seed)
			conf_seed_hash[conf] << seed
		else
			conf_seed_hash[conf] = [seed]
		end
		uniqAlgosToRun << algoToRun
	end

	waitToFinish = false
	unless uniqAlgosToRun.empty?
	# def runAlgosOnCluster(algosToRun, waitToFinish=false, algo = "", output=false, priorityclass = "eh", numCommandsPerJob = 1)
		if oncluster==1 or oncluster=="1"
			runAlgosOnCluster(uniqAlgosToRun, waitToFinish, algo, true, "eh") # -q arrow.q
		elsif oncluster==2 or oncluster=="2"
			runAlgosWithJobserver(uniqAlgosToRun, waitToFinish, algo, true)
		elsif oncluster==3 or oncluster=="3"
			runAlgosOnCluster(uniqAlgosToRun, waitToFinish, algo, true, "eh2")
		elsif oncluster==4 or oncluster=="4"
			waitToFinish = true
			runAlgosOnCluster(uniqAlgosToRun, waitToFinish, "LS"+algo, true, "eh2") # -q arrowtest.q
		elsif oncluster==5 or oncluster=="5"
			waitToFinish = true
			runAlgosOnCluster(uniqAlgosToRun, waitToFinish, "LS"+algo, true, "eh")
		elsif oncluster==6 or oncluster=="6"
			waitToFinish = true
			runAlgosOnCluster(uniqAlgosToRun, waitToFinish, "LS"+algo, true, "eh")
		elsif oncluster==7 or oncluster=="7"
			waitToFinish = false
			runAlgosOnCluster(uniqAlgosToRun, waitToFinish, "LS"+algo, true, "eh")
		elsif oncluster.to_i < 0 
			runAlgosOnCluster(uniqAlgosToRun, waitToFinish, "Ar#{-oncluster.to_i}"+algo, false, "eh ", -oncluster.to_i)
		elsif oncluster==8 or oncluster=="8"
			waitToFinish = false
			runAlgosOnCluster(uniqAlgosToRun, waitToFinish, "One"+algo, true, "eh", 1e9)
		else 
			runAlgosLocally(uniqAlgosToRun)
		end
	end
		
	#=== Read all results if they are ready at this point (i.e. if nothing has to be run using SGE).
	if algosToRun.length == 0 or waitToFinish
		for instance in instanceAndParamsHash.keys
#			puts "Read run results for instance #{instance}" 
			for param_string in instanceAndParamsHash[instance].keys
#				puts "   Read run results for param_string #{param_string}" 
				algorun_config_id = instanceAndParamsHash[instance][param_string]["algorun_config_id"]
				seeds = instanceAndParamsHash[instance][param_string]["seeds"]

				#=== FH changed on Jan 9, 2008: for modified AL database.
				results = [];
				for i in 0...seeds.length
					results << readAlgoResults(algorun_config_id, seeds[i], cutoff_time)
				end
				unless results.class.to_s == "Array"
					raise "Results #{results} must be an array. Instance #{instance}, param_string #{param_string}" 
				end
				unless results[0].class.to_s == "Array"
					p results
					p seeds
					
					p "doing again: readAlgoResults(#{algorun_config_id}, #{seeds})"
					a = readAlgoResults(algorun_config_id, seeds, cutoff_time)
					p "New result"
					p a
					raise "results[0] #{results[0]} must be an array. Instance #{instance}, param_string #{param_string}" 
				end

				instanceAndParamsHash[instance][param_string]["results"] = Hash.new
				for seed in seeds
					res = results.shift

if $fakedCensoringForExactSameResults
					#========================================= FOR DEBUGGING THE PRUNING (CPU time is not absolutely deterministic even with the exact same trajectory :-( )
					#=== If the run is too long and would've timed out, put in the real result.
					if res[1] > real_cutoff_time 
						res[0] = "TIMEOUT"
						res[1] = real_cutoff_time 
					end
end					
					instanceAndParamsHash[instance][param_string]["results"][seed] = res
#					p "changed to res: #{res}"
#					exit
				end
			end
		end
		return true #results ready
	end
	return false #results not ready yet
end


###########################################################################
# The new and improved version of running algorithms: we get a list of entry_param pairs, run them all (as an instanceAndParamsHash) and directly modify the entries.
# If a run has already been performed with a lower cutoff and was successful, then just copy that run and don't rerun it! Of course, also reuse runs with the same cutoff.
###########################################################################
def getAlgoResultsForEntryParamsPairs(algo, entry_params_pairs, cutoff_time, cutoff_length, oncluster=0, db=true)
	cached_state_string = {}
	
	$total_cputime = 0 unless $total_cputime
	$totalEvaluationCount =0 unless $totalEvaluationCount
	#=== Check which of the entry param pairs we actually need to run.
#	puts "Collect runs I need to do."
	entry_params_pairs_to_run = []
	count = 1
	for entry_params in entry_params_pairs
#		p entry_params 
		puts "count #{count}/#{entry_params_pairs.length}" if count.divmod(100)[1]==0
		count += 1
		entry, params = entry_params
		
		if cached_state_string[params]
			state_as_string = cached_state_string[params]
		else
			state_as_string = state_string(params)
			cached_state_string[params] = state_as_string 
		end
		#state_as_string = state_string(params)

#p "entry"
#p entry
#p params 
#$stdout.flush

		if entry["resultForState"].key?(state_as_string)
			#=== If a successful result for a lower cutofftime exist don't rerun it, either.
			lower_cutofftimes = entry["resultForState"][state_as_string].keys
			for lower_cutoff in lower_cutofftimes.sort
				if lower_cutoff > cutoff_time
					# raise "We should NEVER perform a run with shorter cutoff than before"
					if entry["resultForState"][state_as_string][lower_cutoff][0] == "TIMEOUT"
						#=== If the run had a longer runtime and still timed out, copy the run and adapt its runtime.
						entry["resultForState"][state_as_string][cutoff_time] = entry["resultForState"][state_as_string][lower_cutoff].dup
						entry["resultForState"][state_as_string][cutoff_time][1] = cutoff_time
						break
					else
						#=== If the run had a longer runtime and solved the instance: 
						if entry["resultForState"][state_as_string][lower_cutoff][1] <= cutoff_time
							#=== a) It solved the instance with a shorter runtime than we ask for, copy the run.
							entry["resultForState"][state_as_string][cutoff_time] = entry["resultForState"][state_as_string][lower_cutoff].dup
						else
							#=== a) It solved the instance with a longer runtime than we ask for, copy the run and adapt time and solution status.
							entry["resultForState"][state_as_string][cutoff_time] = entry["resultForState"][state_as_string][lower_cutoff].dup
							entry["resultForState"][state_as_string][cutoff_time][1] = cutoff_time
							entry["resultForState"][state_as_string][cutoff_time][0] = "TIMEOUT"
						end
					end
				else
					next if lower_cutoff == cutoff_time #=== No copying needed here.
					if entry["resultForState"][state_as_string][lower_cutoff][0] != "TIMEOUT"
	#					output "Successful lower cutoff: #{lower_cutoff} -- #{entry["resultForState"][state_as_string][lower_cutoff][0]}"
						entry["resultForState"][state_as_string][cutoff_time] = entry["resultForState"][state_as_string][lower_cutoff].dup
						break;
					end
				end
			end

			#=== If the result for this cutofftime exist don't rerun it.
			next if entry["resultForState"][state_as_string].key?(cutoff_time)
			
			#=== If none of the lower runtimes were successful and the current one hasn't been run, then it has to be run.
			entry_params_pairs_to_run << entry_params 
		else
			entry_params_pairs_to_run << entry_params 
		end
	end
	return true if entry_params_pairs_to_run.empty?

#	output "Retrieve runtimes for #{entry_params_pairs_to_run.length} entry_params pairs."
#	puts "Construct instanceAndParamsHash for #{entry_params_pairs_to_run.length} entry_params pairs."
	#=== Construct instanceAndParamsHash to run.
	instanceAndParamsHash = Hash.new

	count = 1
	for entry_params in entry_params_pairs_to_run
		if count.divmod(100)[1]==0
			puts "Got #{count}/#{entry_params_pairs_to_run.length}"
			$stdout.flush
		end
		entry, params = entry_params
#		p params
		if cached_state_string[params]
			param_string = cached_state_string[params]
		else
			param_string = state_string(params)
			cached_state_string[params] = param_string 
		end
		#param_string = state_string(params)
		inst = entry["name"]
		instanceAndParamsHash[inst] = Hash.new unless instanceAndParamsHash.key?(inst)
		
		unless instanceAndParamsHash[inst].key?(param_string)
			instanceAndParamsHash[inst][param_string] = {"qual" => entry["qual"], "reference"=>entry["reference"], "params" => params, "seeds"=>[]}
		end
		instanceAndParamsHash[inst][param_string]["seeds"] << entry["seed"]
		count += 1
	end

#	puts "getAlgoResultsForInstsAndParams."

#	puts "instanceAndParamsHash"
#	p instanceAndParamsHash

	#=== Run everything in the instanceAndParamsHash.
	results_ready = getAlgoResultsForInstsAndParams(algo, instanceAndParamsHash, cutoff_time, cutoff_length, instanceAndParamsHash.keys, oncluster, db)

	if results_ready
		#=== Retrieve those results we had to run.
		for entry_params in entry_params_pairs_to_run
			entry, params = entry_params
			if cached_state_string[params]
				param_string = cached_state_string[params]
			else
				param_string = state_string(params)
				cached_state_string[params] = param_string 
			end
			#param_string = state_string(params)

			inst = entry ["name"]
			seed = entry["seed"]
			
			result = instanceAndParamsHash[inst][param_string]["results"][seed]
			unless result
				puts "ERROR - empty result."
				p instanceAndParamsHash[inst]
				p instanceAndParamsHash[inst][param_string]
				p instanceAndParamsHash[inst][param_string]["results"]

				raise "Empty result for instance #{inst}, seed #{seed}, and param_string #{param_string}"
			end
			unless entry["resultForState"].key?(param_string)
				entry["resultForState"][param_string] = Hash.new
			end
			entry["resultForState"][param_string][cutoff_time] = result

			#=== If the measured runtime is over the cutoff time the base algorithm should have terminated earlier - that's not our fault, so use the cutoff time instead. (Also helping to fix a bug with some entries in the DB.)
			#=== But NEVER count a run with less than 0.1 seconds - in order to account for any potential overhead and in order to not get endless loops with runtimes that are 0.0 seconds.
			$total_cputime += [0.1, [cutoff_time, result[1]].min].max
			
			$totalEvaluationCount += 1
		end
	end
	return results_ready
end

#=== Run numbers for each instances MUST BE INCREASING !
def makeStandardInstanceHash(array_of_detailed_instances, state_as_string)
	instanceHash = Hash.new
	for entry in array_of_detailed_instances
		inst = entry["name"]
		run = entry["run"]
		instanceHash[inst] = {"runs"=>0, "desired_qual"=>entry["qual"], "reference"=>entry["reference"], "result"=>[]} unless instanceHash.key?(inst)
		instanceHash[inst]["runs"] += 1
		censortimes = entry["resultForState"][state_as_string].keys
#		p censortimes
#		p censortimes.max
		instanceHash[inst]["result"] << entry["resultForState"][state_as_string][censortimes.max]
	end
	return instanceHash
end


###########################################################################
# Returns the set of instances and their desired solution qualities contained in a file.
###########################################################################
def getInstanceHash(instanceset_name, algo="none")
	#=== Special case: the algorithm does not take input instances.
	return [{"_#{algo}_dummyInstance"=>{"desired_qual"=> 0, "reference"=>0}}, ["_#{algo}_dummyInstance"]] if instanceset_name == "_"

	#=== Read set of instances to run on. (array of arrays [instance_name, opt])
	instancesSorted = []
	instanceHash = Hash.new
	File.open(instanceset_name){|file| 
	    while line=file.gets;
		raise "Incorrect number of entries #{line.strip.split.length} in #{line}" unless line.strip.split.length >= 2  
		entries = line.chomp.strip.split
		inst = entries[0]
		qual = entries[1]
		reference = entries[2]
		instanceHash[inst] = Hash.new
		instanceHash[inst]["desired_qual"] = qual
		instanceHash[inst]["reference"] = reference
		instancesSorted << inst
	    end
	}
	return [instanceHash, instancesSorted]
end
