function [run_queue, used_worker_ids] = assignRunsToWorkers(run_queue, workspace_savedir, func, worker_ids)
% Assign runs to the free workers.
global allParamStrings; % so we can save it.

idxs_to_be_run = find(run_queue(:,5)==0);
idx_into_idxs_to_be_run = 1;
used_worker_ids = -ones(length(worker_ids),1);

bout(['Assigning jobs to ', num2str(length(worker_ids)), ' workers.\n']);
            
my_assert(~isempty(worker_ids), 'Some workers have to be free when we want to assign new runs.');
my_assert(length(idxs_to_be_run) >= length(worker_ids), 'Invariant: we have a run queued for each worker.');
tic;
input_filenames = {};
for i=1:length(worker_ids)
    worker_id_string = strcat('worker_', num2str(worker_ids(i)));
    
    %= Write input file
    input_filename = strcat(workspace_savedir, '/inputs/', worker_id_string, '.mat');
    idx_into_runqueue = idxs_to_be_run(idx_into_idxs_to_be_run);
    theta_idxs = run_queue(idx_into_runqueue,1);
    pi_idxs = run_queue(idx_into_runqueue,2);
    captimes = run_queue(idx_into_runqueue,3);
    seeds = run_queue(idx_into_runqueue,4);
    typeOfRun = 'executeTargetAlgorithmRun';
    save(input_filename, 'theta_idxs', 'pi_idxs', 'seeds', 'captimes', 'typeOfRun', 'allParamStrings');
%     save(input_filename, 'theta_idxs', 'pi_idxs', 'seeds', 'captimes', 'typeOfRun');
    input_filenames{end+1} = input_filename;
    
    %= Set runs to be done by the worker as running.
    run_queue(idx_into_runqueue,5) = 1;
    used_worker_ids(i) = worker_ids(i);
    
    idx_into_idxs_to_be_run = idx_into_idxs_to_be_run+1;
    %=== The following condition only takes effect if we don't fill the workers.
    if idx_into_idxs_to_be_run > length(idxs_to_be_run)
        break
    end
end
assignTime = toc
run_queue

while ~isempty(input_filenames)
    missing_files = [];
    for i=1:length(input_filenames)
        if ~robust_file_exists(input_filenames{i})
            missing_files{end+1} = input_filenames{i};
        end
    end
    input_filenames = missing_files;
end
    