function simple_demo_gp_censoring
rand('twister',1234);
randn('state',1234);

%=== This simple example is extremely sensitive to the specific censoring
%thresholds used etc.

testfun = @(x) ((x-0.5).^2 + (x-0.25).^3 + 0.5*sin(x*10));
x=[(-1:0.05:1)';1.25];
%    x = x(randperm(length(x)));
xtest = (-2:0.01:2)';
ttest = testfun(xtest);

%=== Set the censoring threshold. For now uniform, but there's no 
%=== requirement for that.
threshold = 0.1 *ones(size(x,1),1); %*randn(size(x,1),1)

%=== Add measurement noise. Very low for now.
%   y=testfun(x) + 0.5*randn(size(x,1),1);
y=testfun(x) + 0.005*randn(size(x,1),1);

%=== Set censoring indicator.
cens = (y>=threshold);
ycens=y;
cens_idx = find(cens==1);
noncens_idx = find(cens==0);
ycens(cens_idx) = threshold(cens_idx);


%=== Learn IVM on uncensored data.
ivmModel_uncens = learnModel(x, y, zeros(size(y,1)), 'ivm');

%=== Learn IVM on censored data.
ivmModel_cens = learnModel(x, ycens, cens, 'ivm');

%=== Use parameters learned for the uncensored version (of course not possible in practice).
%     ivmModel_cens.kern.comp{1}.inverseWidth = ivmModel_uncens.kern.comp{1}.inverseWidth;
%     ivmModel_cens.kern.comp{1}.variance = ivmModel_uncens.kern.comp{1}.variance;
%     ivmModel_cens.kern.comp{2}.variance = ivmModel_uncens.kern.comp{2}.variance;
%     ivmModel_cens = ivmOptimiseIVM(ivmModel_cens, 0);

[ypredUncens, ypredVarUncens] = applyModel(ivmModel_uncens, xtest);
[ypredCens, ypredVarCens] = applyModel(ivmModel_cens, xtest);

perf_eval(x, y, ycens, ttest, xtest, cens, ypredUncens, ypredVarUncens, ypredCens, ypredVarCens);

%=== Learn GPML model on uncensored data.
gpmlModel_uncens = learnModel(x, y, zeros(size(y,1)), 'gpml');
gpmlModel_cens   = learnModel(x, ycens, cens, 'gpml', gpmlModel_uncens.params); %*gpmlModel_uncens.params);


[ypredUncens, ypredVarUncens] = applyModel(gpmlModel_uncens, xtest, 0);
[ypredCens, ypredVarCens] = applyModel(gpmlModel_cens, xtest, 0);

perf_eval(x, y, ycens, ttest, xtest, cens, ypredUncens, ypredVarUncens, ypredCens, ypredVarCens);



function perf_eval(x, y, ycens, ttest, xtest, cens, ypredUncens, ypredVarUncens, ypredCens, ypredVarCens);
    lossCens = -log_likelihood(ttest,ypredCens,ypredVarCens,zeros(length(ttest),1))/length(ttest)
    lossUncens = -log_likelihood(ttest,ypredUncens,ypredVarUncens,zeros(length(ttest),1))/length(ttest)
    rmseCens = sqrt(mean((ypredCens-ttest).^2));
    rmseUncens = sqrt(mean((ypredUncens-ttest).^2));

    figure;
    subplot(1,2,1);
    hold off;
    h = confplot(xtest, ypredCens, 3*sqrt(ypredVarCens));
    hold on; 
    plot(xtest, ypredCens-3*sqrt(ypredVarCens), 'b:'); 
    plot(xtest, ypredCens+3*sqrt(ypredVarCens), 'b:'); 
    censIdx = find(cens==1);
    uncensIdx = find(cens==0);
    plot(x(censIdx),ycens(censIdx),'bx', 'MarkerSize', 5); 

    plot(x(uncensIdx),ycens(uncensIdx),'bo'); 
    plot(xtest,ttest,'r-');
    axis([-2,2,-3,3]);

    subplot(1,2,2);
    hold off;
    h = confplot(xtest, ypredUncens, 3*sqrt(ypredVarUncens));
    hold on; 
    plot(xtest, ypredUncens-3*sqrt(ypredVarUncens), 'b:'); 
    plot(xtest, ypredUncens+3*sqrt(ypredVarUncens), 'b:'); 
    plot(x,y,'bo'); 
    plot(xtest,ttest,'r-');
    axis([-2,2,-3,3]);

