function [theta_idxs, pi_idxs, captimes, seeds, incumbent_theta_idx, eligible_theta_idxs] = determineNewRunsForSingleChallenger(rundata, incumbent_theta_idx, theta_new_idx, al_opts, func, eligible_theta_idxs)
%=== New blocked comparison: select (instance, seed) combination from the ones of the incumbent.
theta_idxs = [];
pi_idxs = [];
captimes = [];
seeds = [];

if theta_new_idx == incumbent_theta_idx
    return;
end
%=== Check how many runs R we already have for the challenger, and request max(1, R) new ones
idxs = find(rundata.used_theta_idxs == theta_new_idx);
upToNumNewRuns = max(1, length(idxs));

%=== Evaluate how challenger compares to incumbent.
[numNew, numInc, numIntersect, newGotAll, obj_new_lb, obj_inc, common_inst_seeds, neededInstSeeds] = compareToIncBlockOnlyInst(theta_new_idx, incumbent_theta_idx, rundata, al_opts, func);

bout(sprintf('Challenger %d: ', theta_new_idx));
if numNew >= 1
    bout(sprintf(' %d shared runs, LB %f (incumbent %d has %f). ', numIntersect, obj_new_lb, incumbent_theta_idx, obj_inc));
    %=== Decide whether to do additional runs.

    %== New one is worse => stop.
    if (obj_new_lb >= obj_inc + 1e-6) % challenger is worse
        bout(sprintf(' Worse -> stopping its evaluation.\n'));
        return;
    end
    
    if newGotAll && (obj_new_lb < obj_inc - 1e-6)
        if ~ismember(theta_new_idx, eligible_theta_idxs)
            eligible_theta_idxs(end+1) = theta_new_idx;
        end
% actually, due to hallucinated data, it is possible for this to happen!        
%         global filename_for_workspace_with_failed_assertion;
%         save(filename_for_workspace_with_failed_assertion);
%         assertion_string = 'It should be impossible that the challenger is better than the incumbent since we just updated the incumbent';
%         error_message = ['Assertion failure: ', assertion_string, '  Saving workspace to ', filename_for_workspace_with_failed_assertion];
%         bout(error_message);
%         error(error_message);
    end
else
    bout(sprintf(' new. '));
end

%=== Make comparison based on common runs and upToNumNewRuns from inc.L \new.L (as many as there are)
perm = randperm(size(neededInstSeeds,1));
if upToNumNewRuns >= length(perm)
    upToNumNewRuns = length(perm);
    eligible_theta_idxs(end+1) = theta_new_idx;
end
L = union(common_inst_seeds, neededInstSeeds(perm(1:min(upToNumNewRuns,length(perm))),:), 'rows');

insts = unique(L(:,1));
if al_opts.adaptive_capping
    bound = get_objective_for_insts(incumbent_theta_idx, rundata, insts, al_opts.overallobj, func, 1) + 1e-3;
else
    bound = inf;
end
bout(sprintf(' Performing %d new run(s), up to a total bound %f on %d instances.\n', min(upToNumNewRuns,length(perm)), bound, length(insts)));

[new_theta_idxs, new_pi_idxs, new_captimes, new_seeds] = determineNewBoundedRuns(rundata, theta_new_idx, L, bound, func, al_opts, 1);
theta_idxs = [theta_idxs; new_theta_idxs];
pi_idxs = [pi_idxs; new_pi_idxs];
captimes = [captimes; new_captimes];
seeds = [seeds; new_seeds];