function [ei_challengers, eiTime, valStats] = select_challengers_with_EI(func, model, means, vars, incumbent_theta_idx, options, valdata, valStats, rundata, learnTime, numEiChallengersNeeded)
global ThetaUniqSoFar;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SELECT F_MIN_SAMPLES FOR THE EI CRITERION
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
switch options.ei_inc
    case 0 %'inc'
        %=== Doing expected improvement OVER THE DISTRIBUTION OF THE INCUMBENT
        s1 = randn('state');
%        f_min_samples = means(incumbent_theta_idx) + sqrt(vars(incumbent_theta_idx)) * randn(options.N_s,1);
        f_min_samples = means(incumbent_theta_idx) + sqrt(vars(incumbent_theta_idx)) * randn(numEiChallengersNeeded,1);
        randn('state', s1);
        %        f_min_samples = f_min_samples(find(f_min_samples>=0));
    case 2 % 'spo'
        idx_inc = find(rundata.used_theta_idxs==incumbent_theta_idx);
        f_min_samples = combineRunObjectives(options.overallobj, rundata.y(idx_inc(:))', func.cutoff);
        if options.logModel == 1 || options.logModel == 3
            f_min_samples = log10(f_min_samples);
        end
    case 3 %'inc_ucb' % thought SKO was using this, but it isn't
        f_min_samples = means(incumbent_theta_idx) + sqrt(vars(incumbent_theta_idx));
    case 4 %'inc_sko' % this is what SKO is using, where the incumbent is the min of mean+std
        f_min_samples = means(incumbent_theta_idx);

    otherwise
        error (strcat('Unknown EI inc criterion options.ei_inc: ', num2str(options.ei_inc)))
end

valStats = plotsForValidatingModel_with_ei(options, model, f_min_samples, valdata, valStats, rundata);
if options.just_valid
    ei_challengers = [];
    eiTime = -1;
    return;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% OPTIMIZATION OF EXPECTED IMPROVEMENT TO SELECT CONFIGURATION TO RUN NEXT.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
bout(sprintf('Optimizing EI at valdata.iteration %d ...\n', [valdata.iteration]));

%=== Prepare everything for a local search to optimize EI.
domains_for_ls = {};
for param=func.cat
    domains_for_ls{param} = 1:func.num_values(param); %#ok<AGROW>
end
cond_params_idxs = func.cond_params_idxs;
parent_param_idxs = func.parent_param_idxs;
ok_parent_value_idxs = func.ok_parent_value_idxs;
if options.ignore_conditionals
    cond_params_idxs = [];
    parent_param_idxs = [];
    ok_parent_value_idxs = {};
end

tic;
localThetaUniqSoFar = ThetaUniqSoFar;
localExpImpCriterion = options.expImpCriterion;
M = length(model.module);
for num_challenger=1:numEiChallengersNeeded
    %=== Select a random set of trees for optimizing this challenger.
    used_tree_idxs = ceil(M * rand(M,1));
%     if options.ei_inc == 0
%         f_min = f_min_samples(num_challenger);
%     else
%         f_min = f_min_samples;
%     end
    my_assert(localExpImpCriterion==7, 'Have to use ei_crit=7 right now: mu-k*std');
    f_min = exprnd(2*log(2)); % using f_min to store k for mu-k*std. sample k from exp. dist with median 1
    used_fmins(num_challenger) = f_min;
    funcHandle = @(theta) neg_ei_of_model(theta, model, localExpImpCriterion, f_min, used_tree_idxs);
    
    if rand < 0.5
        %=== Select LS start configration at random.
        ls_start_config = selectRandomConfigs(func, 1);
        init_idx(num_challenger) = -1;
    else
        %=== Select LS start configuration from one of the previous configurations with top EI.
        negEiThetaUniqSoFar = funcHandle(localThetaUniqSoFar);
        [~, sorted_theta_idxs] = non_order_preserving_sort(negEiThetaUniqSoFar);
        idx_chosen = ceil(rand*ceil(min(numEiChallengersNeeded/2, size(localThetaUniqSoFar,1))));
        init_idx(num_challenger) = idx_chosen;
        ls_start_config = localThetaUniqSoFar(sorted_theta_idxs(idx_chosen),:);
    end
    [ei_challengers(num_challenger,:), neg_ei_of_ls(num_challenger)] = general_basic_local_search_mixed_disc_cont(funcHandle, ls_start_config, func.cat, domains_for_ls, cond_params_idxs, parent_param_idxs, ok_parent_value_idxs);
end

%=== Sort configurations by their EI with the global model.
idx_inc = find(rundata.used_theta_idxs==incumbent_theta_idx);
f_min = combineRunObjectives(options.overallobj, rundata.y(idx_inc(:))', func.cutoff);
if options.logModel == 1
    f_min = log10(f_min);
end
[neg_ei, predmean, predvar] = neg_ei_of_model(ei_challengers, model, options.expImpCriterion, f_min, 1:length(model.module));
for num_challenger=1:numEiChallengersNeeded
    bout(sprintf('Local search %d (k=%f, init_idx=%d) yielded neg. "EI" %f with its model and %f with the global one (%f +/- %f)\n', [num_challenger, used_fmins(num_challenger), init_idx(num_challenger), neg_ei_of_ls(num_challenger), neg_ei(num_challenger), predmean(num_challenger), sqrt(predvar(num_challenger))]));
end
eiTime = toc;
bout(sprintf('Parallel local search for optimizing EI at valdata.iteration %d: took %f s\n', [valdata.iteration, eiTime]));


% %=== Sort configurations by their EI with the global model.
% idx_inc = find(rundata.used_theta_idxs==incumbent_theta_idx);
% f_min = combineRunObjectives(options.overallobj, rundata.y(idx_inc(:))', func.cutoff);
% if options.logModel == 1
%     f_min = log10(f_min);
% end
% [neg_ei, predmean, predvar] = neg_ei_of_model(ei_challengers, model, options.expImpCriterion, f_min, 1:length(model.module));
% [~, neg_ei_sort_idx] = non_order_preserving_sort(neg_ei);
% for i=1:numEiChallengersNeeded
%     num_challenger=neg_ei_sort_idx(i);
%     bout(sprintf('Local search %d (k=%f, init_idx=%d) yielded neg. "EI" %f with its model and %f with the global one (%f +/- %f)\n', [num_challenger, used_fmins(num_challenger), init_idx(num_challenger), neg_ei_of_ls(num_challenger), neg_ei(num_challenger), predmean(num_challenger), sqrt(predvar(num_challenger))]));
% end
% ei_challengers = ei_challengers(neg_ei_sort_idx,:);
% eiTime = toc;
% bout(sprintf('Parallel local search for optimizing EI at valdata.iteration %d: took %f s\n', [valdata.iteration, eiTime]));

%=== Some additional stats for output only.
% [neg_ei, predmean, predvar] = neg_ei_of_model([ei_challengers;ThetaUniqSoFar], model, options.expImpCriterion, f_min, 1:length(model.module));
% [~, neg_ei_sort_idx] = non_order_preserving_sort(neg_ei); 
% [~, mean_sort_idx] = sort(predmean);
% bestmean_idx = mean_sort_idx(1);
% for i=1:length(used_fmins)
%     bout(sprintf('f_min=%f (empirical). used (sampled) f_mins = %f\n', f_min, used_fmins(i)));
% end
% for num_challenger=1:numEiChallengersNeeded
%     bout(sprintf('Rank of pred. mean of config %d with "EI": %f (%f +/- %f) out of %d configs; config with min. pred mean: %f +/- %f, neg. EI=%f\n', [num_challenger, 1+length(find(predmean < predmean(neg_ei_sort_idx(1)) )), predmean(neg_ei_sort_idx(1)), sqrt(predvar(neg_ei_sort_idx(1))), size(ThetaUniqSoFar,1) + size(ei_challengers,1), predmean(bestmean_idx), sqrt(predvar(bestmean_idx)), neg_ei(neg_ei_sort_idx(1)) ]));
% end

% %=== Output predictions for EI challengers.
% for num_challenger=1:size(ei_challengers,1)
%     bout(sprintf('Challenger %d: predicted %f +/- %f, "EI" %f, EI with global model: %d\n', [num_challenger, predmean(num_challenger), sqrt(predvar(num_challenger)), -neg_ei_of_ls(num_challenger), -neg_ei(num_challenger)]));
% end
% num_challenger
function [sorted, sort_idx] = non_order_preserving_sort(array)
%=== Just like sort, but NOT preserving order -- instead random tie breaking
perm = randperm(length(array));
arrayperm = array(perm);
arrayperm = arrayperm(:);
[sorted, sort_idx_perm] = sort(arrayperm);
sort_idx(:,1) = perm(sort_idx_perm);

sorted = reshape(sorted, size(array));
sort_idx = reshape(sort_idx, size(array));