function smbo_worker_as_function(workspace_savedir, worker_id, func)

outfolder = strcat(workspace_savedir, '/outputs/');
% create_clean_directory(outfolder);

infolder = strcat(workspace_savedir, '/inputs/');
% create_clean_directory(infolder);

% logfolder = strcat(workspace_savedir, '/logs/');
% create_clean_directory(logfolder);

% worker_start_file = strcat(workspace_savedir, 'workers.txt');
% starter_fid = fopen(worker_start_file, 'w');

worker_id_string = strcat('worker_', num2str(worker_id));
input_filename = strcat(infolder, '/', worker_id_string, '.mat');
output_filename = strcat(outfolder, '/', worker_id_string, '.mat');

tic;
clearvars -except func input_filename output_filename log_fid % reset to clean state
clearVarTime = toc
%=== Wait until input exists.
input_exists = 0;
numtry = 1;
while ~input_exists
    pause(0.1);
    if exist(input_filename, 'file')
        bout(['Input file exists: ', input_filename, '\n']);
        try
            load(input_filename, 'typeOfRun');
            if strcmp(typeOfRun, 'executeTargetAlgorithmRun')

                load(input_filename, 'theta_idxs', 'pi_idxs', 'seeds', 'captimes', 'allParamStrings');
            elseif strcmp(typeOfRun, 'selectNewChallengers')
                load(input_filename, 'TODO: whatever we need for selecting challengers');
            else
                error(['Unknown type of run: ', typeOfRun]);
            end
            input_exists = 1;
        catch ME
            numtry = numtry+1;
            bout(['Error reading inputs for run: ', ME.message, '\n']);
            if numtry > 100
                ME.rethrow();
            end
        end
    end
end
readInputTime = toc - clearVarTime
bout(['Input file read successfully. Took ', num2str(readInputTime), ' seconds of wall time.\n']);

if strcmp(typeOfRun, 'executeTargetAlgorithmRun')
    % I need the following line since func.funcHandle is lost by saving in compiled Matlab with mcc :(    
    bout(['Doing target algorithm run(s) with theta_idx=', num2str(theta_idxs), ', pi_idx=', num2str(pi_idxs), ', seed= ', num2str(seeds), ', captime= ', num2str(captimes), '\n']);

    beforeAlgoRun = toc;
    [ys, censoreds, runtimes, runlengths, solveds, best_sols] = func.funcHandle(theta_idxs, pi_idxs, seeds, captimes);
    targetRunTime = toc-beforeAlgoRun;
    bout(['Target algorithm run(s) took ', num2str(targetRunTime), ' seconds of wall time: y=', num2str(ys), ', censored=', num2str(censoreds), ', runtime= ', num2str(runtimes), '\n']);

    bout(['Deleting input file ', input_filename, ' to prepare for next run.\n']);
    beforeFileDelete = toc;
    deleteFilesAndReturnWhenFinished({input_filename})
    deleteFileTime = toc-beforeFileDelete;
    bout(['Deleting input file took ', num2str(deleteFileTime), ' seconds of wall time.\n']);

    bout(['Saving result of target algorithm run to ', output_filename, '\n']);
    beforeSaving = toc;
    save(output_filename, 'theta_idxs', 'pi_idxs', 'captimes', 'seeds', 'ys', 'censoreds', 'runtimes', 'runlengths', 'solveds', 'best_sols');
    savingTime = toc-beforeSaving;
    bout(['Saving output file took ', num2str(savingTime), ' seconds of wall time.\n']);

    totalTime = toc;
    bout(['Total time for this input: ', num2str(totalTime), '. Compared to components summing to ', num2str(clearVarTime+readInputTime+targetRunTime+deleteFileTime+savingTime) , '.\n']);

elseif strcmp(typeOfRun, 'selectNewChallengers')
    error(['selectNewChallengers not implemented yet for workers']);

else
    error(['Unknown type of run: ', typeOfRun]);
end