function [incumbent_theta_idx, eligible_theta_idxs] = update_incumbent(rundata, incumbent_theta_idx, eligible_theta_idxs, run_queue, al_opts, func, alTunerTime, iteration, iterationTime)
% Goes through the list of eligible_theta_idxs, and checks if any of them
% is done all their runs from run_queue and is better than the incumbent
% BASED ON ITS RUNS ONLY.  
% We cannot request to have all runs the incumbent has since that is 
% getting additional runs in parallel.

orig_eligible_theta_idxs = eligible_theta_idxs;
for i=1:length(orig_eligible_theta_idxs)
    theta_new_idx = orig_eligible_theta_idxs(i);
    if theta_new_idx == incumbent_theta_idx
        continue
    end
    if ~isempty(intersect(find(run_queue(:,1)==theta_new_idx), find(run_queue(:,5)==0))) 
        continue;
    end
    eligible_theta_idxs = setdiff(eligible_theta_idxs, theta_new_idx); % they got their chance with the current evaluations.
    
    %=== Evaluate how challenger compares to incumbent.
    [numNew, numInc, numIntersect, newGotAll, obj_new_lb, obj_inc, common_inst_seeds, neededInstSeeds] = compareToIncBlockOnlyInst(theta_new_idx, incumbent_theta_idx, rundata, al_opts, func);

    my_assert(numNew >= 1, 'To be eligible for to become incumbent, we need to have a run.');
    bout(sprintf('Update incumbent check for challenger %d: %d common runs, LB %f (incumbent %d has %f). ', theta_new_idx, numIntersect, obj_new_lb, incumbent_theta_idx, obj_inc));

    %=== Decide whether to do additional runs.

    %== New one is worse => stop.
    if (obj_new_lb >= obj_inc - 1e-6) % challenger is worse or same
        bout(sprintf(' Not better, keeping incumbent %d.', incumbent_theta_idx));
    else
        bout(sprintf(' Better -> making %d new incumbent.\n', theta_new_idx));
        incumbent_theta_idx = theta_new_idx;
    end
end
log_new_incumbent(rundata, incumbent_theta_idx, func, al_opts, alTunerTime, iteration, iterationTime);

function incumbent_theta_idx = log_new_incumbent(rundata, theta_new_idx, func, al_opts, alTunerTime, iteration, iterationTime)
global bestconflist;
global allParamStrings;

% Correct, but slow: cens_idx = intersect( find(rundata.used_theta_idxs == theta_new_idx), ...
% intersect( find(rundata.cens), find(rundata.used_captimes < func.cutoff-1e-6) ) );
idx_for_theta = find(rundata.used_theta_idxs == theta_new_idx);
idx_for_theta_cens = idx_for_theta(find(rundata.cens(idx_for_theta)));
cens_idx = idx_for_theta_cens(find(rundata.used_captimes(idx_for_theta_cens) < func.cutoff-1e-6)); 
if ~isempty(cens_idx)
    bout(sprintf('ERROR: got runs for theta becoming incumbent that are censored below the threshold'));
    idx_for_theta
    idx_for_theta_cens
    rundata.used_captimes(idx_for_theta_cens)
    func.cutoff-1e-6
    cens_idx
    error_workspace = 'error_new_inc_has_censored_runs.mat'
    save(error_workspace);
    error('ERROR: got runs for theta becoming incumbent that are censored below the threshold');
end

incumbent_theta_idx = theta_new_idx;
%=== Set bestconflist from here onwards to new incumbent
if length(rundata.used_theta_idxs) > length(bestconflist)
    m = length(rundata.used_theta_idxs) - length(bestconflist);
    bestconflist(end+1:end+m-1) = bestconflist(end);
    bestconflist(length(rundata.used_theta_idxs)) = incumbent_theta_idx;
else
    bestconflist(length(rundata.used_theta_idxs):end) = incumbent_theta_idx;
end

%=== Update history of incumbents and output this one.
new_best_paramstring = allParamStrings{incumbent_theta_idx};
%== Get empirical mean and variance of the mean to output.
data = rundata.y(rundata.used_theta_idxs==incumbent_theta_idx)';
meanInc = combineRunObjectives(al_opts.overallobj, data, func.cutoff);
% if al_opts.logModel == 1 || al_opts.logModel == 3
%     meanInc = log10(meanInc);
% end
stdInc = sqrt( var(data) / length(data) );      % length(data); %
outputNewBest(sum(rundata.runTime), sum(rundata.algoWallTimePerIteration), meanInc, stdInc, incumbent_theta_idx, alTunerTime, iterationTime, new_best_paramstring, length(find(rundata.used_theta_idxs==incumbent_theta_idx)),length(rundata.used_theta_idxs), iteration, al_opts);
