function [finished_runs_with_results, output_filenames_to_delete, used_worker_ids, assigned_runs, output_filenames] = waitForRubyWorkers(iteration, used_worker_ids, assigned_runs, output_filenames, func)

minNumRunsNeeded = length(used_worker_ids);
%=== Wait until input exists.
results = {};
idxs_of_running_done = [];
idxs = 1:length(output_filenames);
while 1
    idxs_left = idxs;
    for num=1:length(idxs_left)
        i=idxs_left(num);
        try
            res = csvread(output_filenames{i});
            if res(end,1)==-1 && res(end,2)==-1 && res(end,3)==-1 && res(end,4)==-1 && res(end,5)==-1
                results{i} = res(1:end-1,:);
                idxs = setdiff(idxs,i);
                idxs_of_running_done(end+1) = i;
                continue;
            end
        catch
        end
    end
    if length(idxs_of_running_done) >= minNumRunsNeeded
        break
    end
    pause(0.1);
end
waitTime = toc;
bout(['Iteration ', num2str(iteration), ': waited ', num2str(waitTime), ' seconds, got ', num2str(length(idxs_of_running_done)), ' new result files.\n']);

finished_runs = assigned_runs(idxs_of_running_done);
results = results(idxs_of_running_done);
output_filenames_to_delete = output_filenames(idxs_of_running_done);

%=== Update used_worker_ids, assigned_runs, output_filenames
idxs_of_running_not_done = setdiff(1:length(used_worker_ids), idxs_of_running_done);
output_filenames = output_filenames(idxs_of_running_not_done);
used_worker_ids = used_worker_ids(idxs_of_running_not_done);
assigned_runs = assigned_runs(idxs_of_running_not_done, :);

%=== Read results and put them into finished_runs_with_results.
finished_runs_with_results = -ones(length(results), length(finished_runs{1})+6);
results
for i=1:length(results)
    finished_runs{i}
    results{i}
    idxs_of_running_done
    readRunResults(results{i}, finished_runs{i}(3),func)
    finished_runs_with_results(i,:)
    [finished_runs{i}, readRunResults(results{i}, finished_runs{i}(3),func)]
    finished_runs_with_results(i,:) = [finished_runs{i}, readRunResults(results{i}, finished_runs{i}(3),func)];
end
% finished_runs_with_results = {};
% for i=1:length(results)
%     for j=1:size(results{i},1)
%         finished_runs_with_results{end+1} = readRunResults(results{i}(j,:), finished_runs{i}(3), func);
%     end
% end

function result = readRunResults(results, censorLimit, func)

N=size(results,1); % actually, here that's just always 1
solveds = -ones(N,1);
censoreds = -ones(N,1);
runtimes = -ones(N,1);
runlengths = -ones(N,1);
best_sols = -ones(N,1);

for i=1:N
    res = results(i,:);
%     seed_out = res(end);    
%     assert(seeds(i) == seed_out, 'must report the correct seed');

    solved = res(end-4);
    if res(end-3) > censorLimit
        solved = 0;
        censoreds(i) = 1;
    end
    solveds(i) = solved;

    if solved == 3 % wrong answer!
        warnstring = strcat(['Wrong answer of target algorithm - check the log for details.\n']);
        warning(warnstring);
        bout(warnstring);

        censoreds(i) = 0;
        runtimes(i) = inf;
        runlengths(i) = inf;
        best_sols(i) = inf;
    else
        if solved == 0 % TIMEOUT
            censoreds(i) = 1;
        else
            censoreds(i) = 0;
        end
        if func.force_runtime_cap_at_limit
            runtimes(i) = min(res(end-3), censorLimit); 
        else
            runtimes(i) = res(end-3);
        end
        runlengths(i) = res(end-2);
        best_sols(i) = res(end-1);
    end
end

runtimes = min(runtimes, 1e7); % don't count wrong answers as infinity.

global runTimeForRunningAlgo;
my_assert(all(runtimes >= -1e-5), 'runtimes have to be >= 0');
my_assert(~isinf(censorLimit), 'infinite censor limits problematic -> counting infinite runtime for runs with wrong answer');
actual_runtimes = min([censorLimit'; runtimes'], [], 1); % to deal with inf in runs with wrong answer
runTimeForRunningAlgo = runTimeForRunningAlgo + sum(actual_runtimes);

switch func.singleRunObjective
    case 'runtime'
        ys = runtimes;
    case 'runlength'    
        ys = runlengths;
    case 'solqual'
        best_sols = min(best_sols, 1e10); % here, 1e10 represents infinity; we first minimize bad cases like that and only break ties on the actual sol. qual.
        ys = best_sols;
        censoreds = zeros(size(ys,1),1);
    otherwise 
        error 'Still have to implement objective functions for single algorithm runs other than runtime, runlength, and solqual.'
end
runtimes = min(runtimes, censorLimit+1e-4); % to deal with inf in runs with wrong answer. Target algo runtime uses rundata.runtimes, so we cannot have a large number in there.

result = [ys, censoreds, runtimes, runlengths, solveds, best_sols];